using System;
using System.IO;
using System.Xml;
using System.Collections;

namespace Dashboard {

	class GaleonBookmarks : BackendSimple {
		XmlDocument         doc;
		XmlNodeList         titles;
		XmlDocument         favicondoc;
		
		public override bool Startup ()
		{
			Name = "Bookmarks";

			string home_dir = Environment.GetEnvironmentVariable ("HOME");
			string path;

			try {
				path = Path.Combine (home_dir, ".galeon/bookmarks.xbel");

				if (!File.Exists (path)){
					path = Path.Combine (home_dir, ".galeon/bookmarks.xml");
					Console.WriteLine ("path: " + path);
					if (!File.Exists (path))
						return false;
				}

				doc = new XmlDocument ();
				doc.Load (path);

				titles = doc.SelectNodes ("//bookmark/title");
				Console.WriteLine ("Bookmarks backend loaded {0} bookmarks.", titles.Count);

				path = Path.Combine (home_dir, ".galeon/favicon_cache.xml");
				Console.WriteLine ("path: " + path);
				if (!File.Exists (path))
					return false;
				favicondoc = new XmlDocument ();
				favicondoc.Load (path);
 				
				// FIXME: need to subscribe to clues that we
				// should try matching against. Probably
				// textblock and sentence_at_point:
				// this.SubscribeToClues ("textblock", "sentence_at_point");

				this.Initialized = true;

				return true;
			} catch {
				return false;
			}
		}

		protected override Match ProcessClueSimple (Clue clue)
		{

			if (! this.Initialized)
				return null;

			if (clue.Text.Length == 0)
				return null;

			string clue_text = clue.Text.ToLower ();

			string html = String.Format ("<table border=0 width=100%><tr><td><h2><u>Bookmarks</u></h2></td></tr></table>");

			int match_count = 0;
			foreach (XmlNode node in titles) {
				string t = node.InnerText;
				string lower = t.ToLower ();

				if (lower.IndexOf (clue_text) == -1)
					continue;

				if (node.ParentNode == null)
					throw new Exception ("Null found");
				if (node.ParentNode.Attributes ["href"] == null) {
					throw new Exception ("Null found222");
				}

                                string uri = node.ParentNode.Attributes ["href"].InnerText;
                                string xpath = "descendant::entry[@url='" + uri + "']";
                                Console.WriteLine (xpath);
                                XmlNode favicon = favicondoc.SelectSingleNode (xpath);
                                string iconuri;
                                if (favicon != null)
                                        iconuri = favicon.Attributes ["favicon"].InnerText;
                                else
                                        iconuri ="internal:bookmark.png";

				html += String.Format (
"<table border=0 cellpadding=0 cellspacing=0>" +
"<tr>" +
"    <td valign=center>" +
+ "        <a href=\"{1}\"><img height=\"48\" width=\"48\" src=\"{2}\" border=0></a>" +
"    </td>" +
"<td>&nbsp;&nbsp;</td>" +
"    <td valign=top>" +
"        <a href=\"{1}\" style=\"text-decoration: none;\">{0}<br>" +
"        <font size=-1 color=#666666>{1}</font></a>" +
"    </td>" +
"</tr>" +
"</table>", t, uri, iconuri);

				match_count ++;
			}

			if (match_count == 0)
				return null;

			return new Match (this, html, 0, String.Format ("bookmark:{0}", clue_text));
		}
	}
}
