//
// backend-bugzilla.cs: A backend which displays bugzilla bugs based
// on their bug numbers.
//
// TODO: Make the bugzilla server(s) configurable.
//       Choose good bug colors.
//
// Nat Friedman <nat@nat.org>
//

using System;
using System.IO;
using System.Collections;
using System.Xml;
using System.Runtime.InteropServices;
using System.Net;

namespace Dashboard {

	class Bugzilla : BackendSimple {

		private const string bugzilla_host = "http://bugzilla.ximian.com/";

		public override bool Startup ()
		{
			Name = "Bugzilla";

			this.SubscribeToClues ("bugzilla");
			this.Initialized = true;

			return true;
		}

		protected override Match ProcessClueSimple (Clue clue)
		{
			XmlDocument xml = this.GetBugzillaXml (clue.Text);
			if (xml == null)
				return null;

			string html = this.XmlBugToHtml (xml);

			return new Match (this, html, 0, String.Format ("Bugzilla Bug:{0}", clue.Text));
		}

		private XmlDocument GetBugzillaXml (string bug) {

			XmlDocument xml = new XmlDocument ();

			HttpWebRequest req = (HttpWebRequest)
				WebRequest.Create (String.Format ("{0}/xml.cgi?{1}", bugzilla_host, bug));
									       
			req.UserAgent = "GNOME Dashboard";
			WebResponse resp = req.GetResponse ();
			StreamReader input = new StreamReader (resp.GetResponseStream ());

			string bugxml = input.ReadToEnd ();
			Console.WriteLine ("BUGZILLA BACKEND Got XML: {0}", bugxml);
			bugxml = bugxml.Substring (bugxml.IndexOf ("<bugzilla"));

			try {
				xml.LoadXml (bugxml);
			} catch {
				Console.WriteLine ("Bugzilla XML is not well-formed!");
				return null;
			}

			return null;
		}

		private string XmlBugToHtml (XmlDocument xml)
		{
			string bug_num, product, summary, owner, status;

			try {
				bug_num = this.GetXmlText (xml, "//bug_id");
				product = this.GetXmlText (xml, "//product");
				summary = this.GetXmlText (xml, "//short_desc");
				summary = summary.Substring (0, Math.Min (summary.Length, 50));
				owner   = this.GetXmlText (xml, "//assigned_to");
				status  = this.GetXmlText (xml, "//bug_status");
			} catch {
				Console.WriteLine ("Could not get bug fields");
				return "";
			}

			string status_color = this.GetStatusColor (status);

			string bug_url = String.Format ("{0}/show_bug.cgi?id={1}", bugzilla_host, bug_num);

			string html = String.Format (
"<table width=100% border=0 cellpadding=1 cellspacing=0><tr>" +
"    <td valign=center>" +
"        <a href=\"{0}\"><img src=\"internal:bug.png\" border=0></a>" +
"    <td valign=top>" +
"        <font size=-3><b>#{1}:</b> {2}<br>" +
"        {3} | <a href=\"mailto:{4}\"><tt>{4}</tt></a> " +
"<font color=\"{5}\"><tt>{6}</tt></font></font>" +
"    </td>" +
"</tr></table>", bug_url, bug_num, summary, product, owner, status_color, status);
			return html;
		}

		private string GetStatusColor (string status)
		{
			switch (status) {

			case "RESOLVED":
				return "#00dd00";

			case "NEEDINFO":
				return "#cad505";

			case "ASSIGNED":
				return "#1438e8";

			default:
				return "#000000";

			}
		}
					      

		private string GetXmlText (XmlDocument xml, string tag)
		{
			XmlNode node;

			node = xml.SelectSingleNode (tag);
			if (node == null)
				return "???";

			return node.InnerText;
		}
	}

} // namespace Dashboard
