using System;
using System.IO;
using System.Collections;

using Dashboard.Index;

namespace Dashboard {

	class DocumentIndex : Backend {
		private static double DECAY = 0.3;

		private IndexManager mgr;

		public override bool Startup ()
		{

			Console.WriteLine ("DocumentIndex backend starting up...");

			Name = "DocumentIndex";

			try {
				mgr = new IndexManager ("local_file");
			}
			catch {
				Console.WriteLine ("DocumentIndex backend: ERROR LOADING.");

				this.Initialized = false;

				return false;
			}

			this.SubscribeToClues ("textblock");
			this.Initialized = mgr.Initialized;

			return this.Initialized;
		}

		public override BackendResult ProcessCluePacket (CluePacket cp)
		{
			ArrayList result_matches = new ArrayList ();

			foreach (Clue clue in cp.Clues) {
				if (! ClueTypeSubscribed (clue))
					continue;

				string clue_text = clue.Text.ToLower ();

				Console.WriteLine ("DocumentIndex: searching for {0}", clue_text);

				ArrayList result_clues; // FIXME: do something with these
				ArrayList matches = mgr.Retrieve (new StringReader (clue_text), 
								  "text/plain", 
								  out result_clues);

				Console.WriteLine ("Got from query: {0}\n", matches.Count);

				if (matches.Count == 0)
					continue;

				foreach (IndexMatch match in matches) {
					// find time decay factor (old files get weighted less)

					long t_1 = DateTime.Now.ToFileTime ();
					long t_0 = match.Source.LastWriteTime;

					// TODO: keep an eye out for clock skew

					long distance = Math.Abs (t_1 - t_0);

					// convert from 100 ns increments to days

					distance /= 10L * 1000L * 1000L * 60L * 60L * 24L;

					double age_factor = Math.Exp (- DECAY * distance);

					match.Relevance *= age_factor;
				}

				matches.Sort ();

				string html = "<table border=0 width=100%><tr><td><h2><u>Related Documents</u></h2></td></tr></table>";

				foreach (IndexMatch match in matches) {
					string icon = Gnome.Icon.LookupByURI (match.Source.URI);
					if (icon == null) {
						icon = "internal:document.png";
					}

					html += String.Format (
"<table width=100% border=0><tr>" +
"    <td valign=center>" +
"        <a href=\"{1}\"><img src=\"{2}\" border=0></a>" +
"    </td><td>&nbsp;&nbsp;</td>" +
"    <td valign=top>" +
"        <a href=\"{1}\" style=\"text-decoration: none;\">{0}<br>" +
"        <font size=-1 color=#666666>{1}</font></a>" +
"    </td>" +
"</tr></table>", PathToFilename (match.Source.URI), match.Source.URI, icon);
				}

				html += "</table>";

				result_matches.Add (new Match (this, html, 0, Name, clue));
			}

			if (result_matches.Count > 0)
				return new BackendResult (result_matches, null);
			else
				return null;
		}

		private string PathToFilename (string path)
		{
			string[] split = path.Split ('/');
			return split [split.Length - 1];
		}
	}
}
