//
// backend-evomail.cs
//
// Author:
//    Nat Friedman <nat@nat.org>
//


using System;
using System.IO;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;

namespace Dashboard {

	class EvoMail : BackendSimple
	{
		Camel.Summary summary;

		// FIXME: We need to autofind the default mail folder path
		const string summary_path = "/home/nat/evolution/local/Play/subfolders/Personal Mail/mbox.ev-summary";

		public override bool Startup ()
		{
			Name = "Evolution Mail";

			try {
				summary = new Camel.Summary (summary_path);
			} catch {
				Console.WriteLine ("Could not load camel summary: " + summary_path);
				return false;
			}

			this.SubscribeToClues ("email");
			this.Initialized = true;

			return true;
		}


		protected override Match ProcessClueSimple (Clue clue)
		{
			ArrayList mails = GetMatchingMails (clue.Text); 
			if (mails == null || mails.Count == 0)
				return null;

			string html = this.GenerateHtml (mails);

			return new Match (this, html, 0, "Evolution Mail");
		}

		// Takes an email addr or a full_name
		ArrayList GetMatchingMails (string text)
		{
			ArrayList mails = new ArrayList ();

			foreach (Camel.MBoxMessageInfo mi in summary.messages) {
				if (mi.from.IndexOf (text) == -1)
					continue;

				mails.Add (mi);
			}

			int n = Math.Min (mails.Count, 10);
			return mails.GetRange (mails.Count - n, n);
		}

		string GenerateHtml (ArrayList mails)
		{
			string html;

			html = "<table border=0 width=100% cellspacing=0>";

			int i = 0;
			foreach (Camel.MBoxMessageInfo mi in mails) {
				string color;

				if (i % 2 == 0)
					color = "#bebabe";
				else
					color = "#ffffff";

				html += this.GenerateHtmlForMessage (mi, color);

				i ++;
			}

			html += "</table>";

			return html;
		}

		string GenerateHtmlForMessage (Camel.MBoxMessageInfo mi, string color)
		{
			// FIXME: Get the unread status and set the image properly
			// FIXME: Figure out what happens when you click the mail, make it work.
			return String.Format ("<tr><td bgcolor=\"{0}\"><img border=0 src=\"internal:mail.png\">" +
					      "</td><td width=100% bgcolor=\"{0}\">{1}</td></tr>",
					      color, mi.subject);
		}
	}
}
