// FOAF backend
// by Edd Dumbill <edd@usefulinc.com>


// TODO: implement basedNear stuff so we can chain latlong clues

using System;
using System.IO;
using System.Net;
using System.Web;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Threading;
using System.Runtime.Remoting.Messaging;

using Dashboard.Rdf.Foaf;

namespace Dashboard {

	class FoafFile : Backend {

		public override bool Startup ()
		{
			Console.WriteLine ("FoafFile backend starting");

			this.SubscribeToClues ("rdfurl", "foafid");
			this.Initialized = true;

			Name = "FoafFile";

			return true;
		}

		private string stripquotes (string txt)
		{
			// remove leading and closing '
			// replace \' with '
			// replace \\ with \
			if (txt.StartsWith ("'"))
				txt = txt.Substring (1);
			if (txt.EndsWith ("'"))
				txt = txt.Substring (0, txt.Length - 1);

			txt = txt.Replace ("\\'", "'");
			return txt.Replace ("\\\\", "\\");
		}

		public BackendResult GetFoafMatches (ArrayList files, ArrayList refs)
		{
			ArrayList matches = new ArrayList ();
			ArrayList newclues = new ArrayList ();
			Store foaf;
			Regex wordregex = new Regex
					("^foaf:(\\w+)\\s+('?.*'?)$");

			foreach (string fname in files) {
				try {
					foaf = new Store (fname);
					Console.WriteLine ("Retrieved RDF file {0}", fname);
				} catch {
					Console.WriteLine ("Ignoring invalid RDF file {0}", fname);
					continue;
				}
				foreach (string s in refs) {
					System.Text.RegularExpressions.Match m =
						wordregex.Match (s);

					if (! m.Success)
						continue;

					string prop = Properties.NAMESPACE_URI + 
						m.Groups[1].ToString ();
					string val = m.Groups[2].ToString ();
					if (val.StartsWith ("'"))
						val = stripquotes(val);

					Console.WriteLine ("Person identified by {0} {1}",
							prop, val);

					string res = foaf.GenerateHtmlCard (prop, val);
					if (res != null)
						matches.Add (new Match (this, res, 0, "FOAF matches"));

					// FIXME: should probably be something better than
					// a string array arraylist
					ArrayList a = foaf.GenerateClues (prop, val);
					foreach (ArrayList c in a) {
						newclues.Add (new Clue ((string)c[0], (string)c[1], 10));
						// Fixme: need to tag the new clues with the
						// clue(s) that triggered them
					}
				}
			}

			return new BackendResult(matches, newclues);
		}

		public override BackendResult ProcessCluePacket (CluePacket cp)
		{
			ArrayList files = new ArrayList ();
			ArrayList refs = new ArrayList ();
			Uri baseuri = null;

			if (! this.Initialized)
				return null;

			foreach (Clue c in cp.Clues) {
				if (c.Text.Length == 0)
					continue;

				// FIXME: this hack assumes that url always comes before
				// content in a block from a browser frontend.  really
				// we ought to examine all the clue content first.
				if (c.Type == "rdfurl")
					files.Add (c.Text);

				if (c.Type == "foafid")
					refs.Add (c.Text);
			}

			return GetFoafMatches (files, refs);
		}
	}
}
