using System;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;

namespace Scraper {

class GoogleHit {
	private static string REGEX_PAT = @"<a href=(.*?)>(.*?)</a><br>(.*)<br>.*?\s+-\s+(\d+\w)\s+.*?<a class=fl href=(.*?)>Cached</a>.*?<a class=fl href=(.*?)>Similar pages</a>";

	private string url;
	private string title;
	private string desc;
	private string size;
	private string cache;
	private string related;

	public GoogleHit (string html) {
		Regex regex = new Regex (REGEX_PAT, RegexOptions.IgnoreCase);

		Match m = regex.Match (html);

		if (m.Success) {
			url     = m.Groups[1].ToString ();
			title   = m.Groups[2].ToString ();
			desc    = m.Groups[3].ToString ();
			size    = m.Groups[4].ToString ();
			cache   = m.Groups[5].ToString ();
			related = "http://www.google.com" + m.Groups[6].ToString ();
		}
		else
			Console.WriteLine ("i don't get it.");
	}

	public override string ToString () {
		string str = "url     [" + url + "]\n" +
		             "title   [" + title + "]\n" +
		             "desc    [" + desc + "]\n" +
		             "size    [" + size + "]\n" +
		             "cache   [" + cache + "]\n" +
		             "related [" + related + "]";

		return str;
	}
}

class Scraper {
	private static string HIT_SEPARATOR  = "<p class=g>";
	private static string HIT_TERMINATOR = "Similar pages</a></font> ";
	private static int    MAX_RESULTS    = 10;

	public static GoogleHit[] QueryGoogle (string queryString) {
		char[] query = queryString.ToCharArray ();

		for (int i=0; i<query.Length; ++i)
			if (Char.IsWhiteSpace (query[i]))
				query[i] = '+';

		string url = "http://www.google.com/search?hl=en&ie=UTF-8&oe=UTF-8&q=" + new string (query) + "&btnG=Google+Search";

		Console.WriteLine (String.Format ("URL: {0}", url));

		HttpWebRequest req = (HttpWebRequest)WebRequest.Create (url);
		req.UserAgent = "mozilla";

		WebResponse resp = req.GetResponse ();

		StreamReader input = new StreamReader (resp.GetResponseStream ());

		string buf;
		int pivot = 0;
		bool feof = false;

	// eat stream until first HIT_SEPARATOR

		do {
			buf = input.ReadLine ();

			if (buf == null)
				feof = true;
			else {
				pivot = buf.IndexOf (HIT_SEPARATOR);
				//				Console.WriteLine (String.Format ("buf: {0}", buf));
			}


		} while (pivot < 0 && !feof);

		if (feof) {
			Console.WriteLine ("EOF");
			return null;
		}

		buf = buf.Substring (pivot + HIT_SEPARATOR.Length);

		GoogleHit[] hits = new GoogleHit[MAX_RESULTS];

		for (int i=0; i<MAX_RESULTS; ++i)
			hits[i] = null;

		for (int i=0; i<MAX_RESULTS && !feof; ++i) {
			pivot = buf.IndexOf (HIT_SEPARATOR);

			while (pivot < 0 && !feof) {
				string line = input.ReadLine ();

				if (line != null)
					buf += line;
				else
					feof = true;

				pivot = buf.IndexOf (HIT_SEPARATOR);
			}

			if (pivot < 0)
				pivot = buf.IndexOf (HIT_TERMINATOR) + HIT_TERMINATOR.Length;

			string html = buf.Substring (0, pivot);

			hits[i] = new GoogleHit (html);

			buf = buf.Substring (pivot + HIT_SEPARATOR.Length);
		}

		return hits;
	}

	public static void Main (string[] args) {
		GoogleHit[] hits = QueryGoogle (args[0]);

		if (hits == null) {
			Console.WriteLine ("no hits");

			return;
		}

		for (int i = 0; i < MAX_RESULTS; ++i) {
			if (hits[i] != null) {
				Console.WriteLine (hits[i]);
				Console.WriteLine ();
			}
		}
	}
}

}
