using System;
using System.IO;
using System.Diagnostics;
using System.Collections;
using System.Runtime.InteropServices;

namespace Dashboard {
	class EncounteredUrl {
		public String url;
		public String context;
		public String from;
		public String fromlink;
	}

	class EncounteredUrls : BackendSimple {
		private ArrayList urls;

		public override bool Startup ()
		{
			Name = "Encountered URLs";

			string home_dir = Environment.GetEnvironmentVariable ("HOME");
			string docs_dir;

			Gnome.VFS.Mime.Init ();

			documents = new ArrayList ();

			try {
				docs_dir = Path.Combine (home_dir, "Documents");

				//
				// Use 'locate' to find all the files in the user's
				// documents directory.
				//
				Process locate = new Process ();
				locate.StartInfo.FileName               = "locate";
				locate.StartInfo.Arguments              = docs_dir;
				locate.StartInfo.UseShellExecute        = false;
				locate.StartInfo.RedirectStandardOutput = true;

				locate.Start ();

				//
				// Read the matches from 'locate' line by line.  Discard
				// the matches which are not in the user's documents
				// directory.
				//
				string doc_line;
				while ((doc_line = locate.StandardOutput.ReadLine ()) != null) {
					if (doc_line.IndexOf (docs_dir) != 0)
						continue;

					Doc doc = new Doc ();
					doc.path = doc_line;
					string[] split = doc_line.Split ('/');
					doc.filename = split [split.Length - 1];

					// Ignore backup and autosave files.
					if (doc.filename [doc.filename.Length - 1] != '~' &&
					    doc.filename [0]                   != '#'     &&
					    doc.path.IndexOf (".thumbnail")    == -1      &&
					    doc.path.IndexOf (".xvpics")    == -1) {
						
						documents.Add (doc);
					}

				}

				Console.WriteLine ("Documents backend loaded {0} documents.", documents.Count);

			} finally {
				//
				// Signal that we are ready to accept clues.
				//
				this.Initialized = true;
			}

			return true;
		}

		protected override Match ProcessClueSimple (Clue clue)
		{
			if (! this.Initialized)
				return null;

			string clue_text = clue.Text.ToLower ();

			//
			// Construct a list of documents that
			// match the clue.
			//
			ArrayList matches = new ArrayList ();
			foreach (Doc doc in documents) {
				if (doc.filename.ToLower ().IndexOf (clue_text) == -1)
					continue;

				matches.Add (doc);
			}

			Console.WriteLine ("About to sort");

			//
			// Sort by mtime.
			//
			matches.Sort (new DocComparer ());

			Console.WriteLine ("sorted");

			string html = String.Format ("<table border=0 width=100%><tr><td><h2><u>Related Documents</u></h2></td></tr></table>");

			foreach (Doc doc in matches) {
				string icon = Gnome.Icon.LookupByURI (doc.path);
				if (icon == null) {
					icon = "internal:document.png";
				}
				Console.WriteLine ("Got icon [{0}] for mime [{0}]", icon);
								
				html += String.Format (
"<table width=100% border=0><tr>" +
"    <td valign=center>" +
"        <a href=\"file://{1}\"><img src=\"{2}\" border=0></a>" +
"    </td><td>&nbsp;&nbsp;</td>" +
"    <td valign=top>" +
"        <a href=\"file://{1}\" style=\"text-decoration: none;\">{0}<br>" +
"        <font size=-1 color=#666666>{1}</font></a>" +
"    </td>" +
"</tr></table>", doc.filename, doc.path, icon);
			}

			html += "</table>";

			return new Match (this, html, 0, "url history", clue);
		}
	}
}
