//
// GNOME Dashboard
//
// clue.cs: The class which represents a clue.
//
// Authors:
//    Nat Friedman <nat@nat.org>
//    Joe Shaw <joe@assbarn.com>
//

using System;
using System.Collections;

namespace Dashboard {

	public class Clue : ICloneable {

		// See doc/cluetypes.txt
		public readonly string     Type;

		// The text of the clue
		public readonly string     Text;

		// The relevance of the clue to what the user's doing (1-10)
		public readonly int        Relevance;

		// True if from a frontend CP, not true if from a cluechain
		public bool Toplevel = false;

		// If cluechained, contains the clues that triggered
		// this one
		public ArrayList TriggeringClues = new ArrayList ();

		public Clue (string type, string text, int relevance, params object[] triggering_clues)
		{
			this.Type           = type;
			this.Text           = text;
			this.Relevance      = relevance;

			if (triggering_clues != null) {
				lock (this.TriggeringClues) {
					foreach (object o in triggering_clues)
						if (o != null)
							this.TriggeringClues.Add ((Clue) o);
				}
			}
		}

		public override string ToString ()
		{
			return String.Format ("Clue Text: {0} Type: {1} Relevance: {2}",
					      this.Text, this.Type, this.Relevance);
		}

		public string ToShortOpaqueString ()
		{
			int hash;

			hash = this.GetClueHash ();
			foreach (Clue c in this.TriggeringClues)
				hash ^= c.GetClueHash ();

			return String.Format ("c{0}", Math.Abs (hash));
		}

		public int GetClueHash () {
			return this.Text.GetHashCode() ^ this.Type.GetHashCode() ^ this.Relevance.GetHashCode();
		}

		public override bool Equals (object o) {
			return this.GetHashCode() == o.GetHashCode();
		}

		public object Clone ()
		{
			return new Clue (this.Type,
					 this.Text,
					 this.Relevance);
		}
	}
}
