//
// GNOME Dashboard
//
// cluepacket.cs: The CluePacket object.  Please see
// doc/cluepacket.txt for information about cluepackets.
//
// Authors: 
//    Nat Friedman <nat@nat.org>
//    Joe Shaw <joe@assbarn.com>
//

using System;
using System.Collections;

namespace Dashboard {
	public delegate void CluePacketNotifyDelegate (CluePacket m);
		
	public class CluePacket : ICloneable {
		public readonly CluePacket Parent;
	
		// The name of the app that sent the cluepacket
		public readonly string Frontend;

		// The context inside the app that sent the cluepacket
		public readonly string Context;

		// Whether or not the triggering context was focused
		public readonly bool Focused;

		// The Xid generatign this cluepacket
		public readonly int Xid;

		public ArrayList Clues = new ArrayList ();

		public CluePacket (CluePacket parent) 
			: this (parent.Frontend, parent.Context, parent.Focused, parent.Xid)
		{
			this.Parent = parent;
			this.Clues  = new ArrayList (parent.Clues);
		}

		public CluePacket (string frontend, string context, bool focused) {
			this.Frontend = frontend;
			this.Context  = context;
			this.Focused  = focused;
		}

		public CluePacket (string frontend, string context, bool focused, int xid) 
			: this (frontend, context, focused)
		{
			this.Xid = xid;
		}

		public bool IsParent (CluePacket child) {
			if (child.Parent == this)
				return true;
			else if (child.Parent != null)
				return IsParent (child.Parent);
			else
				return false;
		}

		public ArrayList GetClues (params string[] clue_types)
		{
			if (clue_types == null) 
				return null;
			else if (clue_types.Length == 0)
				return this.Clues;
			else {
				ArrayList ret = new ArrayList ();
				foreach (Clue c in this.Clues) {
					if (((IList) clue_types).Contains (c.Type))
						ret.Add (c);
				}
				return ret;
			}
		}

		public override string ToString ()
		{
			string s;

			s = String.Format ("Frontend: [{0}]  Context: [{1}]  Focused: [{2}]\n" +
					   "Clues:\n", this.Frontend, this.Context, this.Focused);
			for (int i = 0; i < Clues.Count; i ++)
				s += String.Format ("    {0}\n", this.Clues[i].ToString ());

			return s;
		}

		public object Clone ()
		{
			CluePacket copy = new CluePacket (this.Frontend,
							  this.Context,
							  this.Focused,
							  this.Xid);

			foreach (Clue c in this.Clues)
				copy.Clues.Add (c);

			return copy;
		}
	}
}	
