//
// GNOME Dashboard
//
// match.cs: The Match class.
//

using System;
using System.Collections;

namespace Dashboard {

	public delegate void MatchNotifyDelegate (Match m);

	public class Match : ICloneable{

		// The backend that generated the match
		public readonly Backend Backend;

		// The match data itself; per-backend
		public readonly string Html;

		// The quality of the Match (1-10)
		public readonly int    Score;

		// A simple string describing the match, mainly for
		// debugging output.
		public readonly string Desc;

		// The clues used by the backend to find this match.
		public ArrayList TriggeringClues = new ArrayList ();

		public Match (Backend backend, string html, int score, string desc, params object[] triggering_clues)
		{
			this.Backend = backend;
			this.Html    = html;
			this.Score   = score;
			this.Desc    = desc;

			if (triggering_clues != null)
				foreach (object o in triggering_clues)
					this.TriggeringClues.Add ((Clue) o);
		}

		public override string ToString ()
		{
			return String.Format (
				 "[Backend: [{0}] Score: [{1}] ]",
				 this.Backend.Name, this.Score);
		}

		public string ToShortOpaqueString ()
		{
			int hash;

			hash = this.Html.GetHashCode ();
			foreach (Clue c in this.TriggeringClues)
				hash ^= c.GetClueHash ();

			return String.Format ("m{0}", Math.Abs (hash));
		}

		public object Clone ()
		{
			return new Match (this.Backend,
					  this.Html,
					  this.Score,
					  this.Desc);
		}
	}
}
