//
// GNOME Dashboard
//
// query-context.cs: QueryContext holds all the state for a running
// query triggered by an incoming frontend cluepacket.
//
// Author:
//   Nat Friedman <nat@nat.org>
//
using System;
using System.Collections;

namespace Dashboard {
	public class QueryContext {

		public string     Frontend;
		public string     Context;
		public bool       Focused;
		public int        Xid;

		private ArrayList  clues = new ArrayList ();

		public readonly CluePacket FrontendCluePacket;

		public QueryContext (CluePacket cp)
		{
			this.FrontendCluePacket = cp;

			this.Frontend = cp.Frontend;
			this.Context  = cp.Context;
			this.Focused  = cp.Focused;
			this.Xid      = cp.Xid;

			foreach (Clue c in cp.Clues) {
				c.Toplevel = true;
				this.clues.Add (c);
			}
		}

		public int AddNewClues (ArrayList newclues)
		{
			lock (this) {
				int new_clue_count = 0;

				if (newclues == null)
					return 0;

				foreach (Clue c in newclues)
					if (! this.ClueInList (this.clues, c)) {
						this.clues.Add (c);
						new_clue_count ++;
					}

				return new_clue_count;
			}
		}

		public CluePacket GenerateCurrentCluePacket ()
		{
			lock (this) {
				CluePacket cp;

				cp = new CluePacket (this.Frontend, this.Context, this.Focused, this.Xid);

				foreach (Clue c in this.clues)
					cp.Clues.Add (c);

				return cp;
			}
		}

		private bool ClueInList (ArrayList l, Clue c)
		{
			foreach (Clue cl in l) {
				if (cl.Text == c.Text && cl.Type == c.Type)
					return true;
			}

			return false;
		}

		public override string ToString ()
		{
			string s = "";

			foreach (Clue c in this.clues) {
				if (c.Type == "textblock" || c.Type == "htmlblock" || c.Type == "content")
					s += String.Format ("Clue: [{0}]\n", c.Type);
				else
					s += String.Format ("Clue: [{0}] [{1}]\n", c.Type, c.Text);
			}

			s += "\n";

			return s;
		}

	}
}
